/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/Connection.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron;

/**
 * A <code>Connection</code> is an object from which {@link saffron.Table}s can be
 * accessed.  Every class which implements <code>Connection</code> must
 * implement the <em>public static</em> method <code>Schema getSchema()</code>.
 *
 * @author jhyde
 * @since 10 November, 2001
 * @version $Id: //guest/julian_hyde/saffron/src/main/saffron/Connection.java#2 $
 **/
public interface Connection {
	/**
	 * Returns the schema underlying this connection.  Non-abstract classes
	 * implementing this interface must also provide <code>public static Schema
	 * getSchemaStatic()</code>.
	 **/
	Schema getSchema();

	/**
	 * In theory, this method returns the contents of <code>tableName</code> as
	 * an array; in practice, it is a placeholder recognized by the optimizer
	 * to do something much more efficient. This involves calling <code>{@link
	 * saffron.Schema#getTableForMember}(tableName).{@link
	 * saffron.Table#toRel}(cluster, exp)</code>.
	 **/
	Object contentsAsArray(String tableName);
}


// End Connection.java
