/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/Schema.java#3 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron;
import openjava.ptree.MethodCall;

/**
 * A <code>Schema</code> is a set of {@link saffron.Table}s.
 **/
public interface Schema
{
	/**
	 * Retrieves a {@link saffron.Table} based upon a member access (for example,
	 * <code>salesSchema.emps</code>).
	 **/
	Table getTableForMember(String name);

	/**
	 * Retrieves a {@link saffron.Table} based upon a method call (for example,
	 * <code>salesSchema.empsInDept("SALES")</code>).
	 **/
	Table getTableForMethodCall(MethodCall call);

	/**
	 * Registers all of the rules supported by this schema. Only called by
	 * {@link Planner#registerSchema}.
	 */
	void registerRules(Planner planner) throws Exception;
};

// End Schema.java
