/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/ext/LocaleMin.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.ext;

import saffron.AggregationExtender;

import java.text.Collator;
import java.util.Locale;

/**
 * <code>LocaleMin</code> is an example of a custom aggregation. It evaluates
 * the minimum of a set of values, according to a given locale.
 *
 * @author jhyde
 * @since 10 February, 2002
 * @version $Id: //guest/julian_hyde/saffron/src/main/saffron/ext/LocaleMin.java#2 $
 **/
public class LocaleMin implements AggregationExtender {
	private Collator collator;

	public LocaleMin(Locale locale)
	{
		this.collator = Collator.getInstance(locale);
	}
	private final String lesser(String s, String t)
	{
		if (s != null) {
			if (t == null || collator.compare(s, t) < 0) {
				return s;
			}
		}
		return t;
	}

	// the following methods fulfill the 'AggregationExtender' contract for
	// 'Object' values

	public String aggregate(String v) {
		throw new UnsupportedOperationException();
	}
	public Object start(String v) {
		return null;
	}
	public Object next(String v, Object accumulator) {
		return lesser(v, (String) accumulator);
	}
	public Object merge(String v, Object accumulator0, Object accumulator1) {
		return lesser((String) accumulator0, (String) accumulator1);
	}
	public String result(String v, Object accumulator) {
		return (String) accumulator;
	}
}

// End LocaleMin.java
