/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/ext/ReflectSchema.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.ext;
import openjava.ptree.MethodCall;
import saffron.Planner;
import saffron.Schema;
import saffron.Table;

import java.lang.reflect.Field;

/**
 * <code>ReflectSchema</code> uses reflection to find definitions of
 * tables.
 *
 * @author jhyde
 * @since 10 November, 2001
 * @version $Id: //guest/julian_hyde/saffron/src/main/saffron/ext/ReflectSchema.java#2 $
 **/
public class ReflectSchema implements Schema
{
	// implement Schema
	public Table getTableForMember(String name)
	{
		try {
			Class clazz = getClass();
			Field field = clazz.getField(name);
			Object o = field.get(this);
			if (o instanceof Table) {
				return (Table) o;
			}
			throw new Error(name + " is not a Table");
		} catch (NoSuchFieldException e) {
			return null;
		} catch (IllegalAccessException e) {
			return null;
		}
	}

	// implement Schema
	public Table getTableForMethodCall(MethodCall call)
	{
		// this implementation of Schema doesn't recognize method calls
		return null;
	}

	public void registerRules(Planner planner) throws Exception {
		// this implementation of Schema doesn't have any rules
	}
}

// End ReflectSchema.java
