/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/opt/VisitorRelVisitor.java#1 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.opt;
import openjava.mop.Toolbox;
import openjava.ptree.Expression;
import openjava.ptree.ParseTreeException;
import openjava.ptree.util.ParseTreeVisitor;
import saffron.rel.Rel;
import saffron.util.RelVisitor;

/**
 * Walks over a tree of {@link saffron.rel.Rel}s, walking a {@link ParseTreeVisitor}
 * over every expression in that tree.
 *
 * @author jhyde
 * @since 22 October, 2001
 * @version $Id: //guest/julian_hyde/saffron/src/main/saffron/opt/VisitorRelVisitor.java#1 $
 **/
public class VisitorRelVisitor extends RelVisitor {

	ParseTreeVisitor visitor;

	public VisitorRelVisitor(ParseTreeVisitor visitor)
	{
		this.visitor = visitor;
	}

	// implement RelVisitor
	public void visit(Rel p, int ordinal, Rel parent)
	{
		Expression[] childExps = p.getChildExps();
		try {
			for (int i = 0; i < childExps.length; i++) {
				childExps[i].accept(visitor);
			}
		} catch (ParseTreeException e) {
			throw Toolbox.newInternal(
				e, "while visiting expression " + p);
		}
		p.childrenAccept(this);
	}
}


// End VisitorRelVisitor.java
