/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/OneRow.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel;
import openjava.mop.Environment;
import openjava.mop.OJClass;
import openjava.ptree.util.SyntheticClass;
import saffron.Planner;
import saffron.opt.CallingConvention;
import saffron.opt.Cluster;
import saffron.opt.Cost;
import saffron.rel.java.JavaOneRow;

/**
 * <code>OneRow</code> always returns one row, zero columns.
 *
 * @author jhyde
 * @since 23 September, 2001
 * @version $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/OneRow.java#2 $
 **/
public class OneRow extends Rel {

	/**
	 * Creates an <code>OneRow</code>.
	 *
	 * @param cluster     {@link Cluster} this relational expression belongs to
	 */
	public OneRow(Cluster cluster)
	{
		super(cluster);
	}

	public Object clone()
	{
		return new OneRow(cluster);
	}

	// implement Rel
	protected OJClass deriveRowType()
	{
		Environment env = getEnvironment();
		OJClass clazz = env.lookupClass(env.currentClassName());
		return SyntheticClass.createProject(clazz, new OJClass[0], new String[0]);
	}

	// implement Plan
	public Cost computeSelfCost(Planner planner)
	{
		return planner.makeZeroCost();
	}

	// override Rel
	public Rel changeConvention(Planner planner, int convention)
	{
		switch (convention) {
		case CallingConvention.JAVA:
			return new JavaOneRow(cluster);
		default:
			return null;
		}
	}
}


// End OneRow.java
