/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/convert/ArrayConverter.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel.convert;

import openjava.mop.OJClass;
import openjava.ptree.*;
import saffron.opt.CallingConvention;
import saffron.opt.Cluster;
import saffron.opt.Implementor;
import saffron.rel.Rel;
import saffron.util.Util;

/**
 * An <code>ArrayConverter</code> converts a plan from
 * <code>inConvention</code> to {@link saffron.opt.CallingConvention#ARRAY}.
 **/
class ArrayConverter extends Converter
{
    Variable var_v;

    ArrayConverter(Cluster cluster, Rel child, int inConvention)
    {
        super(cluster, child, inConvention);
    }

    // implement Rel
    public Object clone()
    {
        return new ArrayConverter(cluster, child, inConvention);
    }

    public int getConvention()
    {
        return CallingConvention.ARRAY;
    }

    public static boolean canConvert(int inConvention)
    {
        switch (inConvention) {
        case CallingConvention.COLLECTION:
        case CallingConvention.VECTOR:
            return true;
        default:
            return false;
        }
    }

    // implement Converter
    protected Object implementVector(Implementor implementor, int ordinal)
    {
        switch (ordinal) {
        case -1: // called from parent
            // Generate
            //   saffron.runtime.SaffronUtil.copyInto(
            //     (Vector) <<exp>>, new Type[0])
            OJClass clazz = child.getRowType();	// "Rowtype"
            Object o = implementor.implementChild(this, 0, child);
            return new MethodCall(
                TypeName.forOJClass(
                    OJClass.forClass(saffron.runtime.SaffronUtil.class)),
                "copyInto",
                new ExpressionList(
                    new CastExpression(
                        TypeName.forOJClass(Util.clazzVector),
                        (Expression) o),
                    new ArrayAllocationExpression(
                        clazz,
                        new ExpressionList(Literal.constantZero()))));
        default:
            throw Util.newInternal(
                "implement: ordinal=" + ordinal);
        }
    }

    // implement Converter
    protected Object implementCollection(
        Implementor implementor, int ordinal)
    {
        switch (ordinal) {
        case -1: // called from parent
            // Generate
            //   saffron.runtime.SaffronUtil.copyInto(<<exp>>, new Type[0])
            // if primitive, and
            //   <<exp>>.toArray(new Type[0])
            // otherwise
            OJClass clazz = child.getRowType();	// "Rowtype"
            Expression exp = (Expression) implementor.implementChild(
                this, 0, child);
            return clazz.isPrimitive()
                ? new MethodCall(
                    TypeName.forOJClass(
                        OJClass.forClass(saffron.runtime.SaffronUtil.class)),
                    "copyInto",
                    new ExpressionList(
                        exp,
                        new ArrayAllocationExpression(
                            clazz,
                            new ExpressionList(Literal.constantZero()))))
                 : new MethodCall(
                    exp,
                    "toArray",
                    new ExpressionList(
                        new ArrayAllocationExpression(
                            clazz,
                            new ExpressionList(Literal.constantZero()))));
        default:
            throw Util.newInternal("implement: ordinal=" + ordinal);
        }
    }
}
