/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/convert/EnumerationConverter.java#3 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel.convert;
import openjava.ptree.Expression;
import openjava.ptree.ExpressionList;
import openjava.ptree.MethodCall;
import openjava.ptree.Variable;
import saffron.opt.CallingConvention;
import saffron.opt.Cluster;
import saffron.opt.Implementor;
import saffron.rel.Rel;
import saffron.util.Util;

/**
 * <code>EnumerationConverter</code> converts a plan from
 * <code>inConvention</code> to {@link
 * saffron.opt.CallingConvention#ENUMERATION}.
 *
 * @author jhyde
 * @since 16 December, 2001
 * @version $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/convert/EnumerationConverter.java#3 $
 **/
public class EnumerationConverter extends Converter {
	Variable var_v;

	EnumerationConverter(Cluster cluster, Rel child, int inConvention)
	{
		super(cluster, child, inConvention);
	}

	// implement Rel
	public Object clone()
	{
		return new EnumerationConverter(cluster, child, inConvention);
	}

	public int getConvention()
	{
		return CallingConvention.ENUMERATION;
	}

	public static boolean canConvert(int inConvention)
	{
		switch (inConvention) {
		case CallingConvention.VECTOR:
			return true;
		default:
			return false;
		}
	}

	// implement Converter
	protected Object implementVector(Implementor implementor, int ordinal)
	{
		switch (ordinal) {
		case -1: // called from parent
			// Generate
			//   <<exp>>.elements()
			Expression exp = (Expression) implementor.implementChild(
				this, 0, child);
			return new MethodCall(
				exp, "elements", new ExpressionList());
		default:
			throw Util.newInternal("implement: ordinal=" + ordinal);
		}
	}
}


// End EnumerationConverter.java
