/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/iter/IterConcatenate.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel.iter;

import openjava.mop.OJClass;
import openjava.ptree.*;
import saffron.Planner;
import saffron.opt.CallingConvention;
import saffron.opt.Cluster;
import saffron.opt.Cost;
import saffron.opt.Implementor;
import saffron.rel.Rel;
import saffron.rel.Union;
import saffron.util.Util;

/**
 * <code>IterConcatenate</code> concatenates several iterators. It is an
 * iterator implementation of {@link Union}.
 **/
public class IterConcatenate extends Union
{
    public IterConcatenate(Cluster cluster, Rel[] inputs)
    {
        super(cluster, inputs, true /*all*/);
    }
    public Object clone()
    {
        return new IterConcatenate(cluster, inputs);
    }
    public int getConvention()
    {
        return CallingConvention.ITERATOR;
    }
    public Cost computeSelfCost(Planner planner)
    {
        double dRows = getRows(),
            dCpu = 0,
            dIo = 0;
        return planner.makeCost(dRows, dCpu, dIo);
    }
    public Object implement(Implementor implementor, int ordinal)
    {
        switch (ordinal) {
        case -1:
            // Generate
            //   new CompoundIterator(
            //     new Iterator[] {<<input0>>, ...})
            ExpressionList exps = new ExpressionList();
            for (int i = 0; i < inputs.length; i++) {
                Expression exp = (Expression) implementor.implementChild(
                    this, i, inputs[i]);
                exps.add(exp);
            }
            return new AllocationExpression(
                OJClass.forClass(saffron.runtime.CompoundIterator.class),
                new ExpressionList(
                    new ArrayAllocationExpression(
                        Util.clazzIterator,
                        new ExpressionList(null),
                        new ArrayInitializer(
                            exps))));

        default:
            throw Util.newInternal(
                "implement: ordinal=" + ordinal);
        }
    }
}
