/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/java/JavaFilter.java#3 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel.java;

import openjava.ptree.Expression;
import openjava.ptree.IfStatement;
import openjava.ptree.StatementList;
import saffron.opt.CallingConvention;
import saffron.opt.Cluster;
import saffron.opt.Implementor;
import saffron.rel.Rel;
import saffron.util.Util;

/**
 * <code>JavaFilter</code> implements {@link saffron.rel.Filter} in code.
 **/
public class JavaFilter extends saffron.rel.Filter
{
	public JavaFilter(Cluster cluster, Rel child, Expression condition)
	{
		super(cluster, child, condition);
	}

	public Object clone()
	{
		return new JavaFilter(
			cluster, (Rel) child.clone(), Util.clone(condition));
	}

	// implement Rel
	public int getConvention()
	{
		return CallingConvention.JAVA;
	}

	// implement Rel
	public Object implement(Implementor implementor, int ordinal)
	{
		switch (ordinal) {
		case -1: // called from parent
			return implementor.implementChild(this, 0, child);

		case 0: // called from child
			// Generate
			//	 if (condition) {
			//	   <<parent-handler>>
			//	 }
			StatementList stmtList = implementor.getStatementList();
			StatementList ifBody = new StatementList();
			Expression condition2 = implementor.translate(this, condition);
			stmtList.add(
				new IfStatement(condition2, ifBody));
			implementor.bind(this, child);
			implementor.generateParentBody(this, ifBody);
			return null;

		default:
			throw Util.newInternal(
				"implement: ordinal=" + ordinal);
		}
	}
}

// End JavaFilter.java
