/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/rel/java/JavaProject.java#3 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
//
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.rel.java;

import openjava.ptree.*;
import saffron.opt.CallingConvention;
import saffron.opt.Cluster;
import saffron.opt.Implementor;
import saffron.rel.Project;
import saffron.rel.Rel;
import saffron.util.Util;

/**
 * <code>JavaProject</code> implements {@link Project} as code.
 **/
public class JavaProject extends Project
{
	public JavaProject(
		Cluster cluster, Rel child, Expression[] exps, String[] fieldNames,
		int flags)
	{
		super(cluster, child, exps, fieldNames, flags);
		Util.assert(child.getConvention() == CallingConvention.JAVA);
	}

	public Object clone()
	{
		return new JavaProject(
			cluster, Util.clone(child), Util.clone(exps),
			Util.clone(fieldNames), getFlags());
	}

	// implement Rel
	public int getConvention()
	{
		return CallingConvention.JAVA;
	}

	// implement Rel
	public Object implement(Implementor implementor, int ordinal)
	{
		switch (ordinal) {
		case -1: // called from parent
			return implementor.implementChild(this, 0, child);

		case 0: // called from child
			StatementList stmtList = implementor.getStatementList();
			implementor.generateParentBody(this, null);
			return null;

		default:
			throw Util.newInternal(
				"implement: ordinal=" + ordinal);
		}
	}

	/**
	 * Generates the initializer for the variable which holds the current row
	 * of this project.
	 *
	 * If the row-type V is complex, generate
	 *	  V v = new V(exp, ...);
	 *	 <<parent-handler(v)>>
	 *
	 * If the row-type is simple, generate
	 *	 V v = exp;
	 *	 <<parent-handler(v)>>
	 *
	 * Todo: map relations to expressions (not variables), so we
	 * don't have to create new objects:
	 *	 T1 v1 = exp1;
	 *	 T2 v2 = exp2;
	 *	 <<parent-handler(v1,v2)>>
	 *
	 * Or at least, create a holder object outside the loop, and
	 * just assign into it:
	 *	 V v = new V();
	 *	 <<loop>> {
	 *	   v.v1 = exp1;
	 *	   v.v2 = exp2;
	 *	   <<parent-handler(v)>>
	 *	 }
	 *
	 */
	public Expression implementSelf(Implementor implementor) {
		if ((flags & flagBoxed) != flagBoxed) {
			// simple row-type, hence "V v = exp;"
			return implementor.translate(this, exps[0]);
		} else {
			// complex row-type, hence "V v = new V(exp, ...);"
			ExpressionList args = new ExpressionList();
			for (int i = 0; i < exps.length; i++) {
				args.add(implementor.translate(this, exps[i]));
			}
			return new AllocationExpression(getRowType(), args);
		}
	}
}

// End JavaProject.java
