/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/runtime/SyntheticObject.java#1 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
//
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
//
// jhyde, 23 April, 2002
*/
package saffron.runtime;

import java.lang.reflect.Member;
import java.lang.reflect.Field;

/**
 * <code>SyntheticObject</code> is a base class for all objects built 'on the
 * fly' by the saffron system.
 *
 * For example, if you write<blockquote>
 *
 * <pre>for (i in select {emp.firstName, emp.lastName} from emps as emp) {
 *     ...
 * }</pre>
 *
 * then <code>i</code>'s type will be a class, generated by the saffron
 * compiler, something like this:<blockquote>
 *
 * <pre>class saffron.runtime.Dummy_a01bc65 extends {@link SyntheticObject} {
 *     public String firstName;
 *     public String lastName;
 * }</pre>
 *
 * @author jhyde
 * @since 23 April, 2002
 * @version $Id: //guest/julian_hyde/saffron/src/main/saffron/runtime/SyntheticObject.java#1 $
 **/
public abstract class SyntheticObject {

	/**
	 * Returns the fields of this object, in the order they were specified
	 * in the original query.
	 */
	public Field[] getFields() {
		return getClass().getDeclaredFields();
	}

	/**
	 * Returns the value of a given field.
	 */
	public Object getFieldValue(int i) {
		try {
			Field field = getFields()[i];
			return field.get(this);
		} catch (IllegalArgumentException e) {
			return null;
		} catch (IllegalAccessException e) {
			return null;
		}
	}
}

// End SyntheticObject.java
